sp_configure 'show advanced options', 1
go

reconfigure
go

sp_configure 'clr enabled', 1
go

reconfigure
go

sp_configure 'xp_cmdshell', 1
go

reconfigure
go

if exists (select name from master..sysdatabases where name = 'ClrPerfDB')
	drop database ClrPerfDB
go

create database ClrPerfDB COLLATE Latin1_General_100_BIN2
go
alter database ClrPerfDB add filegroup ClrPerfDB_mod contains memory_optimized_data
go
-- adapt filename as needed
alter database ClrPerfDB add file (name='ClrPerfDB_mod', filename='t:\data\ClrPerfDB_mod') to filegroup ClrPerfDB_mod
go

use ClrPerfDB
go

if object_id ('t_Products') is not null
	drop table t_Products
go

create table t_Products( PID int NOT NULL primary key NONCLUSTERED HASH WITH (BUCKET_COUNT = 128), Pname nvarchar(256), InventoryCost int, StartupCost int)
WITH ( MEMORY_OPTIMIZED = ON , DURABILITY = SCHEMA_AND_DATA )
go

if object_id ('t_SalesForecast') is not null
	drop table t_SalesForecast
go

create table t_SalesForecast( PID int not null, WeekDate smalldatetime not null, DemandQty int
,CONSTRAINT PK_SalesForecast PRIMARY KEY NONCLUSTERED HASH 
( PID, WeekDate ) WITH (BUCKET_COUNT=32768))
WITH ( MEMORY_OPTIMIZED = ON , DURABILITY = SCHEMA_AND_DATA )
go

/*
if object_id ('clr_ProdSched_MemLow_DataHigh_SO') is not null
	drop procedure clr_ProdSched_MemLow_DataHigh_SO
go

if exists (select * from sys.assemblies where name = 'ProdSchedInproc')
	drop assembly ProdSchedInproc
go

create assembly ProdSchedInproc from 'c:\temp\ProdSchedServer.dll'
go

create procedure clr_ProdSched_MemLow_DataHigh_SO(@startDate smalldatetime, @numWeeks int, @K int)
as external name ProdSchedInproc.ClrPerf.ProdSched_MemLow_DataHigh_SO
go
*/

--populate Products
declare @i int
set @i = 1
while @i <= 100
begin
	insert t_Products values(@i,'Junk' + cast(@i as nvarchar),1,75)
	set @i = @i+1
end
go

--populate SalesForecast
declare @i int, @j int, @date smalldatetime, @qty int
set @i = 1
set @qty = 100
while @i <= 100
begin
	set @j = 0
	set @date = '04/01/2004'
	while @j < 205
	begin
		insert t_SalesForecast values( @i, @date, @qty % 50)
		set @j = @j+1
		set @date = DATEADD( Week, 1, @date )
		set @qty = @qty + 13
	end
	set @i = @i+1
end
go

select * from t_Products -- 100 rows
select * from t_SalesForecast -- 20500 rows

DROP PROCEDURE tsql_ProductionScheduling
go

Create type Fk as TABLE 
( k int not null, val int not null, j int not null,
PRIMARY KEY NONCLUSTERED HASH (k,val,j) WITH (BUCKET_COUNT=32))
WITH ( MEMORY_OPTIMIZED = ON )

Create type CIJ as TABLE 
( pid int not null, i int not null, j int not null, C int not null,
PRIMARY KEY NONCLUSTERED HASH (pid, i, j) WITH (BUCKET_COUNT=32))
WITH ( MEMORY_OPTIMIZED = ON )

CREATE TYPE Forecast as TABLE 
( Product nvarchar(256) not null, Period DateTime not null, Quantity int,
PRIMARY KEY NONCLUSTERED HASH (Product, Period) WITH (BUCKET_COUNT=32))
WITH ( MEMORY_OPTIMIZED = ON )
go

Create type FkAgg as TABLE 
( k int not null, val int not null,
PRIMARY KEY NONCLUSTERED HASH (k,val) WITH (BUCKET_COUNT=32))
WITH ( MEMORY_OPTIMIZED = ON )
go

-- As of CTP2, no subquery
CREATE PROCEDURE tsql_ProductionScheduling
	@startDate DateTime, @numWeeks int, @pid int
WITH NATIVE_COMPILATION, EXECUTE AS OWNER, SCHEMABINDING
AS
BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = N'us_english')

DECLARE @Fk dbo.Fk
DECLARE @Fk_final dbo.Fk
DECLARE @CIJ dbo.CIJ
DECLARE @Forecast dbo.Forecast

/*
CREATE TABLE #Fk (k int, val int, j int)
CREATE TABLE #Fk_final (k int, val int, j int)
CREATE TABLE #CIJ ( pid int, i int, j int, C int )
CREATE TABLE #Forecast ( Product nvarchar(256), Period DateTime, Quantity int )
*/

DECLARE @i int, @j int, @n int, @K int, @h int
--DECLARE @pid int
DECLARE @lastValue int
--These are hard coded, but need to come as paramters to storedproc
SET @n = @numWeeks
------------------------------------------------------------------
SELECT @K = StartupCost, @h = InventoryCost FROM dbo.t_Products WHERE PID = @pid
SET @i = 1
WHILE ( @i < @n+1 )
BEGIN
	SET @j = @i+1
	INSERT @CIJ VALUES (@pid,@i,@j,@K)
	SET @lastValue = @K
	SET @j = @j+1
	WHILE( @j <= @n+1 )
	BEGIN
		INSERT @CIJ SELECT PID, @i, @j, @lastValue + (@h * DemandQty * (@j-1-@i) )
			FROM dbo.t_SalesForecast
			WHERE PID=@pid AND WeekDate >= @startDate
			AND WeekDate = DATEADD(Week,@j-2,@startDate)

		SELECT @lastValue = C FROM @CIJ
			WHERE i=@i AND j=@j
	SET @j = @j+1
	END
SET @i = @i+1
END
--SELECT * FROM @CIJ
--Calculate Fk
SET @n = @numWeeks
SET @k = @n+1
INSERT @Fk VALUES( @k, 0, @k+1)
SET @k = @k-1
WHILE( @k >= 1  )
BEGIN
	INSERT @Fk
	SELECT @k, MIN( t.C + f.val ), t.j
	FROM @CIJ t, @Fk f
	WHERE t.i = @k AND t.j > @k AND f.k = t.j
	GROUP BY t.j
SET @k = @k-1
END

/* Subqueries not supprted
INSERT @Fk_final
SELECT Z.k, Z.val, Y.j
FROM
	(SELECT k, MIN(val) AS val FROM @Fk GROUP BY k) Z
	JOIN @Fk Y ON Z.k=Y.k AND Z.val=Y.val
--SELECT * FROM @Fk_final
*/
-- Replacement:
DECLARE @Fk_agg dbo.FkAgg
INSERT @Fk_agg
SELECT k, MIN(val) AS val FROM @Fk GROUP BY k

INSERT @Fk_final
SELECT Z.k, Z.val, Y.j
FROM
	@Fk_agg Z
	JOIN @Fk Y ON Z.k=Y.k AND Z.val=Y.val
-- end of rewrite

DECLARE @product nvarchar(256)
SELECT @product = Pname FROM dbo.t_Products WHERE PID = @pid
--SET @startIndex = 1
DECLARE @sum int, @optimal_j int
DECLARE @week DateTime
DECLARE @period int
SET @k = 1
SET @n = @numWeeks
WHILE ( @k <= @n )
BEGIN
	SET @period = @k
	SELECT @optimal_j = j FROM @Fk_final WHERE k=@k
	SET @sum = 0
	WHILE( @k < @optimal_j )
	BEGIN
		SELECT @sum = @sum + DemandQty FROM dbo.t_SalesForecast 
			WHERE WeekDate = DATEADD(Week,@k-1,@startDate)
			AND PID = @pid
		SELECT @week = WeekDate FROM dbo.t_SalesForecast
			WHERE WeekDate = DATEADD(Week,@period-1,@startDate)
			AND PID = @pid
	SET @k = @k+1
	END
	INSERT @Forecast SELECT @product, @week, @sum
END

SELECT Product, Period, Quantity FROM @Forecast
END
go
