
USE AdventureWorksDW2008R2;
GO

CREATE PARTITION FUNCTION [ByOrderDateMonthPF](int) AS RANGE RIGHT 
FOR VALUES (
    20050701, 20050801, 20050901, 20051001, 20051101, 20051201, 
    20060101, 20060201, 20060301, 20060401, 20060501, 20060601, 
    20060701, 20060801, 20060901, 20061001, 20061101, 20061201, 
    20070101, 20070201, 20070301, 20070401, 20070501, 20070601, 
    20070701, 20070801, 20070901, 20071001, 20071101, 20071201, 
    20080101, 20080201, 20080301, 20080401, 20080501, 20080601, 
    20080701, 20080801, 20080901, 20081001, 20081101, 20081201
) 
GO

CREATE PARTITION SCHEME [ByOrderDateMonthRange] 
AS PARTITION [ByOrderDateMonthPF] 
ALL TO ([PRIMARY]) 
GO

-- Create a partitioned version of the FactResellerSales table
CREATE TABLE [dbo].[FactResellerSalesPart]( 
    [ProductKey] [int] NOT NULL, 
    [OrderDateKey] [int] NOT NULL, 
    [DueDateKey] [int] NOT NULL, 
    [ShipDateKey] [int] NOT NULL, 
    [ResellerKey] [int] NOT NULL, 
    [EmployeeKey] [int] NOT NULL, 
    [PromotionKey] [int] NOT NULL, 
    [CurrencyKey] [int] NOT NULL, 
    [SalesTerritoryKey] [int] NOT NULL, 
    [SalesOrderNumber] [nvarchar](20) NOT NULL, 
    [SalesOrderLineNumber] [tinyint] NOT NULL, 
    [RevisionNumber] [tinyint] NULL, 
    [OrderQuantity] [smallint] NULL, 
    [UnitPrice] [money] NULL, 
    [ExtendedAmount] [money] NULL, 
    [UnitPriceDiscountPct] [float] NULL, 
    [DiscountAmount] [float] NULL, 
    [ProductStandardCost] [money] NULL, 
    [TotalProductCost] [money] NULL, 
    [SalesAmount] [money] NULL, 
    [TaxAmt] [money] NULL, 
    [Freight] [money] NULL, 
    [CarrierTrackingNumber] [nvarchar](25) NULL, 
    [CustomerPONumber] [nvarchar](25) NULL
) ON ByOrderDateMonthRange(OrderDateKey);
GO

-- Using simple or bulk logged recovery mode, and then the TABLOCK 
-- hint on the target table of the INSERTSELECT is a best practice
-- because it causes minimal logging and is therefore much faster.
ALTER DATABASE AdventureWorksDW2008R2 SET RECOVERY SIMPLE;
GO

-- Copy the data from the FactResellerSales into the new table
INSERT INTO dbo.FactResellerSalesPart WITH(TABLOCK)
SELECT * FROM dbo.FactResellerSales;
GO

--Msg 8657, Level 17, State 5, Line 1
--Could not get the memory grant of 146888 KB because it exceeds the maximum configuration limit in workload group 'default' (2) and resource pool 'default' (2).  Contact the server administrator to increase the memory usage limit.

-- had to do this before query would run
USE [master]
GO

/****** Object:  WorkloadGroup [default]    Script Date: 2/13/2011 2:13:17 PM ******/
ALTER WORKLOAD GROUP [default] WITH(group_max_requests=0, 
		importance=Medium, 
		request_max_cpu_time_sec=0, 
		request_max_memory_grant_percent=25, 
		request_memory_grant_timeout_sec=0, 
		max_dop=0)
GO

alter resource governor reconfigure
go

USE AdventureWorksDW2008R2;
GO

-- Create the columnstore index
CREATE NONCLUSTERED COLUMNSTORE INDEX [csindx_FactResellerSalesPart]
ON [FactResellerSalesPart]
( 
    [ProductKey], 
    [OrderDateKey], 
    [DueDateKey], 
    [ShipDateKey], 
    [ResellerKey], 
    [EmployeeKey], 
    [PromotionKey], 
    [CurrencyKey], 
    [SalesTerritoryKey], 
    [SalesOrderNumber], 
    [SalesOrderLineNumber], 
    [RevisionNumber], 
    [OrderQuantity], 
    [UnitPrice], 
    [ExtendedAmount], 
    [UnitPriceDiscountPct], 
    [DiscountAmount], 
    [ProductStandardCost], 
    [TotalProductCost], 
    [SalesAmount], 
    [TaxAmt], 
    [Freight], 
    [CarrierTrackingNumber], 
    [CustomerPONumber] 
);

-- metadata
select * from sys.indexes
where name = 'csindx_FactResellerSalesPart';

-- one row per column in index, per partition
select * from sys.column_store_dictionaries;
-- counts, min/max, size
select * from sys.column_store_segments;

-- Archive 2005 Partition
select OBJECT_NAME(p.object_id) as TableName, p.partition_id, p.data_compression, p.data_compression_desc, rg.state, rg.state_description, 
    rg.total_rows as SegmentRows,  p.rows as PartitionRows, rg.size_in_bytes as 'Size'
    from sys.partitions p
        inner join sys.column_store_row_groups rg
            on p.object_id = rg.object_id and p.partition_number = rg.partition_number 
    where p.object_id = object_id('dbo.FactResellerSalesPart');

alter Index csindx_FactResellerSalesPart on dbo.FactResellerSalesPart
    REBUILD PARTITION = 2 
        WITH (DATA_COMPRESSION = COLUMNSTORE_ARCHIVE);

select * from sys.indexes



-- Use it, uses columnstore
SELECT SalesTerritoryKey, SUM(ExtendedAmount) AS SalesByTerritory
FROM FactResellerSalesPart
GROUP BY SalesTerritoryKey;
GO

-- An aggregate query joining two tables
-- Consider a star-join query that calculates how many of product 215 we sold each quarter. The fact table named FactResellerSalesPart is partitioned on the OrderDateKey column. The dimension table named DimDate links to the fact table through a primary key  foreign key relationship on the date key.
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;
GO

SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;

-- segment elimination
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215 AND f.OrderDateKey >= 20050101 AND f.OrderDateKey < 20060101
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;


-- Additionally, there will be queries that may only target a single table. In such cases, SQL Server tries to leverage the power of batch execution and column store technology to speed up query execution as well.

-- A simple aggregate query on a single table
-- The fact table is FactResellerSalesPart and this table is partitioned on the OrderDateKey column. The following query returns the number of rows and the number of orders.
-- Uses row mode
SELECT COUNT(*) AS NumberOfRows, 
    COUNT(DISTINCT(f.SalesOrderNumber)) AS NumberOfOrders
FROM dbo.FactResellerSalesPart AS f;
GO

-- Determine Size
SELECT SUM(on_disk_size_MB) AS TotalSizeInMB
FROM
(
   (SELECT SUM(css.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_segments AS css
        ON css.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSalesPart') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
  UNION ALL
   (SELECT SUM(csd.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_dictionaries AS csd
        ON csd.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSalesPart') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
) AS SegmentsPlusDictionary


-- Hint
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f WITH (INDEX(csindx_FactResellerSalesPart))
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;


-- simulate a larger table
UPDATE STATISTICS FactResellerSalesPart WITH ROWCOUNT = 10000000, PAGECOUNT = 1000000
GO

DBCC FREEPROCCACHE
GO

-- uses batch mode
SELECT SalesTerritoryKey, SUM(ExtendedAmount) AS SalesByTerritory
FROM FactResellerSalesPart
GROUP BY SalesTerritoryKey;
GO

-- look for new iterator
select top 1000 * 
into FactResellerSalesPartSubset 
from dbo.FactResellerSalesPart; 


-- Batch Hash Table Build iterator in Batch mode
select p.DueDateKey, count(*) 
from dbo.FactResellerSalesPart p, dbo.FactResellerSalesPartSubset m 
where p.SalesOrderLineNumber = m.SalesOrderLineNumber 
group by p.DueDateKey 
order by p.DueDateKey; 

-- new hint
SELECT SalesTerritoryKey, SUM(ExtendedAmount) AS SalesByTerritory
FROM FactResellerSalesPart
GROUP BY SalesTerritoryKey
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

-- does not use batch mode
select p.DueDateKey, count(*) as DueDateCount
from dbo.FactResellerSalesPartSubset m 
left outer join dbo.FactResellerSalesPart p
on p.SalesOrderLineNumber = m.SalesOrderLineNumber 
group by p.DueDateKey 
order by p.DueDateKey; 
GO

-- drop table
DROP TABLE FactResellerSalesPartSubset
DROP TABLE FactResellerSalesPart
DROP PARTITION SCHEME [ByOrderDateMonthRange]
DROP PARTITION FUNCTION [ByOrderDateMonthPF]

/*
select * from sys.partitions 
where data_compression_desc = 'COLUMNSTORE';
select * from sys.columns;

select * from sys.column_store_segments
where column_id = 2
order by column_id;
*/