
use AdventureWorksDW2008R2
go

SELECT * INTO dbo.FactInternetSalesNewCCI FROM dbo.FactInternetSales;
GO

INSERT dbo.FactInternetSalesNewCCI
SELECT * FROM dbo.FactInternetSales;
GO 4

CREATE CLUSTERED COLUMNSTORE INDEX PK_FactInternetSalesNew
ON dbo.FactInternetSalesNewCCI
GO

SELECT * INTO dbo.FactInternetSalesNewNCI FROM dbo.FactInternetSales;
GO

INSERT dbo.FactInternetSalesNewNCI
SELECT * FROM dbo.FactInternetSales;
GO 4

/* Do this with GUI, needs a column list
CREATE COLUMNSTORE INDEX Idx_FactInternetSalesNew
ON dbo.FactInternetSalesNewNCI
GO
*/

--orig query
-- 119 rows affected
select ProductAlternateKey, CalendarYear, SUM(SalesAmount) as SumSales
from FactInternetSalesNewCCI fact
join DimDate
  on fact.OrderDateKey = DateKey
join DimProduct
  on DimProduct.ProductKey = fact.ProductKey
where CalendarYear BETWEEN 2006 AND 2007
  and ProductAlternateKey LIKE 'BK%'
group by ProductAlternateKey, CalendarYear
go

-- left outer joins
-- attempt left outer join, query rewritten as inner join in SQL2012
select ProductAlternateKey, CalendarYear, SUM(SalesAmount) as SumSales
from FactInternetSalesNewCCI fact
left outer join DimDate
  on fact.OrderDateKey = DateKey
left outer join DimProduct
  on DimProduct.ProductKey = fact.ProductKey
where CalendarYear BETWEEN 2006 AND 2007
  and ProductAlternateKey LIKE 'BK%'
group by ProductAlternateKey, CalendarYear
go

-- left outer join without predicate
-- 288 rows affected
-- rewritten as right outer join which uses batch mode in SQL2012
select ProductAlternateKey, CalendarYear, SUM(SalesAmount) as SumSales
from FactInternetSalesNewCCI fact
left outer join DimDate
  on fact.OrderDateKey = DateKey
left outer join DimProduct
  on DimProduct.ProductKey = fact.ProductKey
group by ProductAlternateKey, CalendarYear
go

-- right outer join rewritten as 1 left outer join and one inner
-- 673 rows and warning "Warning: Null value is eliminated by an aggregate or other SET operation."
-- batch mode for join
select ProductAlternateKey, CalendarYear, SUM(SalesAmount) as SumSales
from FactInternetSalesNewCCI fact
right outer join DimDate
  on fact.OrderDateKey = DateKey
right outer join DimProduct
  on DimProduct.ProductKey = fact.ProductKey
group by ProductAlternateKey, CalendarYear
go

-- left outer join without predicate, one dim table, dimension table as first table
-- 504 rows affected
-- uses row mode
select ProductAlternateKey, SUM(SalesAmount) as SumSales
from DimProduct 
left outer join FactInternetSalesNewCCI fact
  on DimProduct.ProductKey = fact.ProductKey
group by ProductAlternateKey
order by SumSales desc
go

-- right outer join rewritten as 1 left outer join and one inner
-- 673 rows and warning "Warning: Null value is eliminated by an aggregate or other SET operation."
-- row mode for join
select ProductAlternateKey, CalendarYear, SUM(SalesAmount) as SumSales
from FactInternetSalesNewNCI fact
right outer join DimDate
  on fact.OrderDateKey = DateKey
right outer join DimProduct
  on DimProduct.ProductKey = fact.ProductKey
group by ProductAlternateKey, CalendarYear
go

-- left outer join without predicate, one dim table, dimension table as first table
-- 504 rows affected
-- uses row mode
select ProductAlternateKey, SUM(SalesAmount) as SumSales
from DimProduct 
left outer join FactInternetSalesNewNCI fact
  on DimProduct.ProductKey = fact.ProductKey
group by ProductAlternateKey
order by SumSales desc
go
