
-- Look at all non-batch mode rewrites
-- Almost no rewrites needed anymore

use ContosoRetailDW -- 2012
use ContosoRetailDW -- 2014

-- outer joins
select prod.ProductName, sum(sales.SalesAmount)
    from dbo.DimProduct prod
        right outer join dbo.FactOnlineSales sales
            on sales.ProductKey = prod.ProductKey
    group by prod.ProductName
    order by prod.ProductName

-- In
select (select store.StoreName from dbo.DimStore store where store.StoreKey = sales.StoreKey) as 'Store', sum(sales.SalesAmount) as 'Store Sales'
    from dbo.FactOnlineSales sales
        where sales.ProductKey  in ( select prod.ProductKey from dbo.DimProduct prod )
    group by sales.StoreKey
    order by sales.StoreKey
 
-- Exists 
select (select store.StoreName from dbo.DimStore store where store.StoreKey = sales.StoreKey) as 'Store', sum(sales.SalesAmount) as 'Store Sales'
    from dbo.FactOnlineSales sales
        where EXISTS ( select prod.ProductKey from dbo.DimProduct prod where sales.ProductKey = prod.ProductKey )
    group by sales.StoreKey
    order by sales.StoreKey

-- Not In
-- Sales without Luxury Products
select (select store.StoreName from dbo.DimStore store where store.StoreKey = sales.StoreKey) as 'Store', sum(sales.SalesAmount) as 'Non-Luxury Store Sales'
    from dbo.FactOnlineSales sales
        where sales.ProductKey  not in ( select prod.ProductKey from dbo.DimProduct prod where prod.ClassID = 3 )
    group by sales.StoreKey
    order by sales.StoreKey
go

-- Union all
-- Let us create a view on the totality of the purchases 
create view dbo.vPurchases as
select [SalesQuantity], [TotalCost]
    from dbo.FactOnlineSales
union all
select [SalesQuantity], [TotalCost]
    from dbo.FactSales 
go
 
-- Lets get the totals out of this view
select sum([SalesQuantity]) as TotalSales, sum([TotalCost]) as TotalCosts
    from dbo.vPurchases

-- Scalar aggregates
-- Total count of the sales
select count(*)
    from dbo.FactOnlineSales

-- Multiple distinct scalar aggregates 
-- In 2014 this uses row mode
select count( distinct StoreKey ) as 'Distinct Stores'
     , count( distinct CustomerKey ) as 'Distinct Customers'
     , count( distinct CurrencyKey ) as 'Distinct Currencies'
from dbo.FactOnlineSales sales
    inner join dbo.DimPromotion prom
        on sales.PromotionKey = prom.PromotionKey
where prom.DiscountPercent = 0
group by sales.LoadDate
