
-- Meant to enumerate items (elements and attributes) 
-- Over a series of XML documents

-- There only appears to be one input_relation, calculator, and stats_collection per-document
WITH xEvents AS
(SELECT object_name AS xEventName,
CAST (event_data AS xml) AS xEventData
FROM sys.fn_xe_file_target_read_file
('C:\Temp\TracesAll\Cardinality*.xel', NULL, NULL, NULL)), -- XEvent session should have file target, change to represent your location
cEvents AS
(SELECT xEventData.value('(/event/action[@name="sql_text"]/value)[1]','varchar(max)') SQLStatement,
xEventData.value('(/event/data[@name="stats_collection_id"]/value)[1]','bigint') StatsCollectionID,
xEventData.query('(/event/data[@name="input_relation"]/value/*)[1]') InputRelation,
xEventData.query('(/event/data[@name="calculator"]/value/*)[1]') Calculator,
xEventData.query('(/event/data[@name="stats_collection"]/value/*)[1]') StatsCollection,
xEventData as EntireEvent
FROM xEvents
)
-- local-name requires singleton or empty sequence
select *
from cEvents
GO

-- Get all elements or attributes
WITH xEvents AS
(SELECT object_name AS xEventName,
CAST (event_data AS xml) AS xEventData
FROM sys.fn_xe_file_target_read_file
('C:\Temp\TracesAll\Cardinality*.xel', NULL, NULL, NULL)), -- XEvent session should have file target, change to represent your location
cEvents AS
(SELECT xEventData.value('(/event/action[@name="sql_text"]/value)[1]','varchar(max)') SQLStatement,
xEventData.value('(/event/data[@name="stats_collection_id"]/value)[1]','bigint') StatsCollectionID,
xEventData.query('(/event/data[@name="input_relation"]/value/*)[1]') InputRelation,
xEventData.query('(/event/data[@name="calculator"]/value/*)[1]') Calculator,
xEventData.query('(/event/data[@name="stats_collection"]/value/*)[1]') StatsCollection,
xEventData as EntireEvent
FROM xEvents
)
-- local-name requires singleton or empty sequence
select distinct t.c.value('local-name(.)','varchar(100)') as Name
from cEvents
cross apply Calculator.nodes('//*') as t(c)  -- all elements *, all attributes @*
order by Name

SELECT object_name AS xEventName,
CAST (event_data AS xml) AS xEventData
FROM sys.fn_xe_file_target_read_file
('C:\Temp\TracesAll\Cardinality*.xel', NULL, NULL, NULL) 
where event_data like '%Residual%'


/* Input Relation
   Element Names (10)
AggFuncInfo
ArithmeticInfo
CompInfo
ConstInfo
ConvertInfo
IdentifierInfo
LogicalInfo
Operator
OpProjectInfo
StatsCollection
   Attribute Names (13)
AggType
Card
ClassNo
ColumnName
CompareOp
Id
LogicalOp
Name
Operation
TableAlias
TableName
Type
Value
*/

/* Calculator 
   ElementNames (5)
CalculatorList
DistinctCountCalculator
FilterCalculator
JoinCalculator
SubCalculator
   AttributeNames (25)
CalculatorFailed
CalculatorName
ColumnName
ComparisonType
Conjunction
CoveringStatDensity
CoveringStatId
Guesses
MultiColumnStat
NumAsymmetricEdgesLeft
NumAsymmetricEdgesRight
NumEqEdges
OverridingColumns
ProbeColumn
ProbeColumnTable
Role
Selectivity
SelectivityBeforeAdjustmentForOverPopulatedDimension
SideToDelegate
SingleColumnStat
StatId
TableName
UnionAllHistograms
UseAverageFrequency
UseBaseCardinality
*/

/* StatsCollection 
   ElementNames
LoadedStats
StatsCollection
StatsInfo
   AttributeNames(8)
Card
DbId
Id
Name
ObjectId
StatsId
TableAlias
TableName
*/

-- find a single attribute or element, which row
;WITH xEvents AS
(SELECT object_name AS xEventName,
CAST (event_data AS xml) AS xEventData
FROM sys.fn_xe_file_target_read_file
('C:\Temp\Cardinality*.xel', NULL, NULL, NULL)), -- XEvent session should have file target, change to represent your location
cEvents AS
(SELECT xEventData.value('(/event/action[@name="sql_text"]/value)[1]','varchar(max)') SQLStatement,
xEventData.value('(/event/data[@name="stats_collection_id"]/value)[1]','bigint') StatsCollectionID,
xEventData.query('(/event/data[@name="input_relation"]/value/*)[1]') InputRelation,
xEventData.query('(/event/data[@name="calculator"]/value/*)[1]') Calculator,
xEventData.query('(/event/data[@name="stats_collection"]/value/*)[1]') StatsCollection,
xEventData.value('((/event/data[@name="calculator"]/value/*)[1]/*/@SelectivityBeforeAdjustmentForOverPopulatedDimension)[1]', 'varchar(10)') SelectAdj,
xEventData.value('((/event/data[@name="calculator"]/value/*)[1]/*/@Selectivity)[1]', 'varchar(10)') Selectivity,
xEventData as EntireEvent
FROM xEvents
)
select * 
from cEvents
--where cast(Calculator as nvarchar(max)) like '%Guesses%'
where cast(Calculator as nvarchar(max)) like '%SelectivityBeforeAdjustmentForOverPopulatedDimension%'