/*
Assume that DBAs who are performing regularly index rebuild operations
have to be controlled through Resource Governor for their maximum IOPS usage. 

CREATE a resource pool and workload group for DBAs
We use a very high value for MAX_IOPS_PER_VOLUME so that we are
currently running unlimited.
*/
CREATE RESOURCE POOL DbaPool WITH
(
 MAX_IOPS_PER_VOLUME = 100000  -- there is also MIN_IOPS_PER_VOLUME
)
GO
CREATE WORKLOAD GROUP DbaGroup
USING DbaPool
GO

-- 
-- Create a new Classifier Function for Resource Governor
CREATE FUNCTION dbo.fn_Classifier()
RETURNS SYSNAME WITH SCHEMABINDING
AS
BEGIN
 DECLARE @GroupName SYSNAME
 
 IF SUSER_NAME() = 'SomeDBA'
 BEGIN
  SET @GroupName = 'DbaGroup'
 END
 ELSE
 BEGIN
  SET @GroupName = 'Default'
 END
 
 RETURN @GroupName;
END
GO

-- Register the Classifier Function within Resource Governor
ALTER RESOURCE GOVERNOR WITH
(
 CLASSIFIER_FUNCTION = dbo.fn_Classifier
)
GO
-- Reconfigure Resource Governor
ALTER RESOURCE GOVERNOR RECONFIGURE
GO

-- Connect as "SomeDBA"
-- You can check workload group using column group_id in the DMV sys.dm_exec_sessions 

-- Create a simple Non-Clustered Index
USE ContosoRetailDW
go

CREATE NONCLUSTERED INDEX idx_DateKey ON FactOnlineSales(DateKey)
GO

-- We are currently not restricted on our IOPS. 
-- Monitor SQL Server:Resource Pool Stats:Disk Write IO/Sec
ALTER INDEX idx_DateKey ON FactOnlineSales REBUILD
GO

-- The following picture shows the output from my system.
-- As you can see the Index Rebuild takes around 100 IOPS. 
-- In the next step restrict the Resource Pool DbaPool to only 50 IOPS:
ALTER RESOURCE POOL DbaPool WITH
(
 MAX_IOPS_PER_VOLUME = 50
)
GO

-- A rebuild issued now would take less IOPS
ALTER INDEX idx_DateKey ON FactOnlineSales REBUILD
GO
